searchData={"items":[{"type":"module","title":"Abyss","doc":"Abyss is a modern, pure Elixir UDP socket server","ref":"Abyss.html"},{"type":"type","title":"Abyss.options/0","doc":"","ref":"Abyss.html#t:options/0"},{"type":"function","title":"Abyss.resume/1","doc":"Resume a suspended server. This will reopen the listening port, and resume the acceptance of new\nconnections","ref":"Abyss.html#resume/1"},{"type":"function","title":"Abyss.start_link/1","doc":"Starts a `Abyss` instance with the given options. Returns a pid\nthat can be used to further manipulate the server via other functions defined on\nthis module in the case of success, or an error tuple describing the reason the\nserver was unable to start in the case of failure.","ref":"Abyss.html#start_link/1"},{"type":"function","title":"Abyss.stop/2","doc":"Synchronously stops the given server, waiting up to the given number of milliseconds\nfor existing connections to finish up. Immediately upon calling this function,\nthe server stops listening for new connections, and then proceeds to wait until\neither all existing connections have completed or the specified timeout has\nelapsed.","ref":"Abyss.html#stop/2"},{"type":"function","title":"Abyss.suspend/1","doc":"Suspend the server. This will close the listening port, and will stop the acceptance of new\nconnections. Existing connections will stay connected and will continue to be processed.\n\nThe server can later be resumed by calling `resume/1`, or shut down via standard supervision\npatterns.\n\nIf this function returns `:error`, it is unlikely that the server is in a useable state\n\nNote that if you do not explicitly set a port (or if you set port to `0`), then the server will\nbind to a different port when you resume it. This new port can be obtained as usual via the\n`listener_info/1` function. This is not a concern if you explicitly set a port value when first\ninstantiating the server","ref":"Abyss.html#suspend/1"},{"type":"type","title":"Abyss.transport_module/0","doc":"A module implementing `Abyss.Transport` behaviour","ref":"Abyss.html#t:transport_module/0"},{"type":"type","title":"Abyss.transport_options/0","doc":"A keyword list of options to be passed to the transport module's `listen/2` function","ref":"Abyss.html#t:transport_options/0"},{"type":"module","title":"Abyss.Echo","doc":"","ref":"Abyss.Echo.html"},{"type":"function","title":"Abyss.Echo.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Abyss.Echo.html#child_spec/1"},{"type":"function","title":"Abyss.Echo.start_link/1","doc":"","ref":"Abyss.Echo.html#start_link/1"},{"type":"behaviour","title":"Abyss.Handler","doc":"`Abyss.Handler` defines the behaviour required of the application layer of a Abyss server.\n\n# Example\n\nA simple example of a Hello World server is as follows:\n\n```elixir\ndefmodule HelloWorld do\n  use Abyss.Handler\n\n  @impl Abyss.Handler\n  def handle_connection(socket, state) do\n    Abyss.Transport.UDP.send(socket, \"Hello, World\")\n    {:close, state}\n  end\nend\n```\n\nAnother example of a server that echoes back all data sent to it is as follows:\n\n```elixir\ndefmodule Echo do\n  use Abyss.Handler\n\n  @impl Abyss.Handler\n  def handle_data(data, socket, state) do\n    Abyss.Transport.UDP.send(socket, data)\n    {:continue, state}\n  end\nend\n```\n\nNote that in this example there is no `c:handle_connection/2` callback defined. The default implementation of this\ncallback will simply return `{:continue, state}`, which is appropriate for cases where the client is the first\nparty to communicate.\n\nAnother example of a server which can send and receive messages asynchronously is as follows:\n\n```elixir\ndefmodule Messenger do\n  use Abyss.Handler\n\n  @impl Abyss.Handler\n  def handle_data(msg, _socket, state) do\n    IO.puts(msg)\n    {:continue, state}\n  end\n\n  def handle_info({:send, msg}, {socket, state}) do\n    Abyss.Transport.UDP.send(socket, msg)\n    {:noreply, {socket, state}, socket.read_timeout}\n  end\nend\n```\n\nNote that in this example we make use of the fact that the handler process is really just a GenServer to send it messages\nwhich are able to make use of the underlying socket. This allows for bidirectional sending and receiving of messages in\nan asynchronous manner.\n\nYou can pass options to the default handler underlying `GenServer` by passing a `genserver_options` key to `Abyss.start_link/1`\ncontaining `t:GenServer.options/0` to be passed to the last argument of `GenServer.start_link/3`.\n\nPlease note that you should not pass the `name` `t:GenServer.option/0`. If you need to register handler processes for\nlater lookup and use, you should perform process registration in `handle_connection/2`, ensuring the handler process is\nregistered only after the underlying connection is established and you have access to the connection socket and metadata\nvia `Abyss.Transport.UDP.peername/1`.\n\nFor example, using a custom process registry via `Registry`:\n\n```elixir\n\ndefmodule Messenger do\n  use Abyss.Handler\n\n  @impl Abyss.Handler\n  def handle_connection(socket, state) do\n    {:ok, {ip, port}} = Abyss.Transport.UDP.peername(socket)\n    {:ok, _pid} = Registry.register(MessengerRegistry, {state[:my_key], address}, nil)\n    {:continue, state}\n  end\n\n  @impl Abyss.Handler\n  def handle_data(data, socket, state) do\n    Abyss.Transport.UDP.send(socket, data)\n    {:continue, state}\n  end\nend\n```\n\nThis example assumes you have started a `Registry` and registered it under the name `MessengerRegistry`.\n\n# When Handler Isn't Enough\n\nThe `use Abyss.Handler` implementation should be flexible enough to power just about any handler, however if\nthis should not be the case for you, there is an escape hatch available. If you require more flexibility than the\n`Abyss.Handler` behaviour provides, you are free to specify any module which implements `start_link/1` as the\n`handler_module` parameter. The process of getting from this new process to a ready-to-use socket is somewhat\ndelicate, however. The steps required are as follows:\n\n1. Abyss calls `start_link/1` on the configured `handler_module`, passing in a tuple\nconsisting of the configured handler and genserver opts. This function is expected to return a\nconventional `GenServer.on_start()` style tuple. Note that this newly created process is not\npassed the connection socket immediately.\n2. The raw `t:Abyss.Transport.socket()` socket will be passed to the new process via a\nmessage of the form `{:abyss_received, listener_socket, server_config, acceptor_span,\nstart_time}`.\n3. Your implenentation must turn this into a `to::inet.socket()` socket by using the\n`Abyss.Transport.UDP.new/3` call.\n4. Your implementation must then call `Abyss.Transport.UDP.handshake/1` with the socket as the\nsole argument in order to finalize the setup of the socket.\n5. The socket is now ready to use.\n\nIn addition to this process, there are several other considerations to be aware of:\n\n* The underlying socket is closed automatically when the handler process ends.\n\n* Handler processes should have a restart strategy of `:temporary` to ensure that Abyss does not attempt to\nrestart crashed handlers.\n\n* Handler processes should trap exit if possible so that existing connections can be given a chance to cleanly shut\ndown when shutting down a Abyss server instance.\n\n* Some of the `:connection` family of telemetry span events are emitted by the\n`Abyss.Handler` implementation. If you use your own implementation in its place it is\nlikely that such spans will not behave as expected.","ref":"Abyss.Handler.html"},{"type":"function","title":"Abyss.Handler.genserver_impl/0","doc":"","ref":"Abyss.Handler.html#genserver_impl/0"},{"type":"callback","title":"Abyss.Handler.handle_close/1","doc":"This callback is called when the underlying socket is closed by the remote end; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas already been closed by the time this callback is called. The return value is ignored.\n\nThis callback is not called if the connection is explicitly closed via `Abyss.Transport.UDP.close/1`, however it\nwill be called in cases where `handle_connection/2` or `handle_data/3` return a `{:close, state}` tuple.","ref":"Abyss.Handler.html#c:handle_close/1"},{"type":"callback","title":"Abyss.Handler.handle_data/2","doc":"This callback is called whenever client data is received after `c:handle_connection/2` or `c:handle_data/3` have returned an\n`{:continue, state}` tuple. The data received is passed as the first argument, and handlers may choose to interact\nsynchronously with the socket in this callback via calls to various `Abyss.Transport.UDP` functions.\n\nThe value returned by this callback causes Abyss to proceed in one of several ways:\n\n* Returning `{:close, state}` will cause Abyss to close the socket & call the `c:handle_close/2` callback to\nallow final cleanup to be done.\n* Returning `{:continue, state}` will cause Abyss to switch the socket to an asynchronous mode. When the\nclient subsequently sends data (or if there is already unread data waiting from the client), Abyss will call\n`c:handle_data/3` to allow this data to be processed.\n* Returning `{:continue, state, timeout}` is identical to the previous case with the\naddition of a timeout. If `timeout` milliseconds passes with no data being received or messages\nbeing sent to the process, the socket will be closed and `c:handle_timeout/2` will be called.\nNote that this timeout is not persistent; it applies only to the interval until the next message\nis received. In order to set a persistent timeout for all future messages (essentially\noverwriting the value of `read_timeout` that was set at server startup), a value of\n`{:persistent, timeout}` may be returned.\n* Returning `{:error, reason, state}` will cause Abyss to close the socket & call the `c:handle_error/3` callback to\nallow final cleanup to be done.","ref":"Abyss.Handler.html#c:handle_data/2"},{"type":"callback","title":"Abyss.Handler.handle_error/2","doc":"This callback is called when the underlying socket encounters an error; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas already been closed by the time this callback is called. The return value is ignored.\n\nIn addition to socket level errors, this callback is also called in cases where `handle_connection/2` or `handle_data/3`\nreturn a `{:error, reason, state}` tuple, or when connection handshaking (typically TLS\nnegotiation) fails.","ref":"Abyss.Handler.html#c:handle_error/2"},{"type":"callback","title":"Abyss.Handler.handle_shutdown/1","doc":"This callback is called when the server process itself is being shut down; it should perform any cleanup required\nas it is the last callback called before the process backing this connection is terminated. The underlying socket\nhas NOT been closed by the time this callback is called. The return value is ignored.\n\nThis callback is only called when the shutdown reason is `:normal`, and is subject to the same caveats described\nin `c:GenServer.terminate/2`.","ref":"Abyss.Handler.html#c:handle_shutdown/1"},{"type":"callback","title":"Abyss.Handler.handle_timeout/1","doc":"This callback is called when a handler process has gone more than `timeout` ms without receiving\neither remote data or a local message. The value used for `timeout` defaults to the\n`read_timeout` value specified at server startup, and may be overridden on a one-shot or\npersistent basis based on values returned from `c:handle_connection/2` or `c:handle_data/3`\ncalls. Note that it is NOT called on explicit `Abyss.Transport.UDP.recv/3` calls as they have\ntheir own timeout semantics. The underlying socket has NOT been closed by the time this callback\nis called. The return value is ignored.","ref":"Abyss.Handler.html#c:handle_timeout/1"},{"type":"function","title":"Abyss.Handler.handler_impl/0","doc":"","ref":"Abyss.Handler.html#handler_impl/0"},{"type":"type","title":"Abyss.Handler.handler_result/0","doc":"The value returned by `c:handle_connection/2` and `c:handle_data/3`","ref":"Abyss.Handler.html#t:handler_result/0"},{"type":"type","title":"Abyss.Handler.timeout_options/0","doc":"The possible ways to indicate a timeout when returning values to Abyss","ref":"Abyss.Handler.html#t:timeout_options/0"},{"type":"module","title":"Abyss.Logger","doc":"Logging conveniences for Abyss servers\n\nAllows dynamically adding and altering the log level used to trace connections\nwithin a Abyss server via the use of telemetry hooks. Should you wish\nto do your own logging or tracking of these events, a complete list of the\ntelemetry events emitted by Abyss is described in the module\ndocumentation for `Abyss.Telemetry`.","ref":"Abyss.Logger.html"},{"type":"function","title":"Abyss.Logger.attach_logger/1","doc":"Start logging Abyss at the specified log level. Valid values for log\nlevel are `:error`, `:info`, `:debug`, and `:trace`. Enabling a given log\nlevel implicitly enables all higher log levels as well.","ref":"Abyss.Logger.html#attach_logger/1"},{"type":"function","title":"Abyss.Logger.detach_logger/1","doc":"Stop logging Abyss at the specified log level. Disabling a given log\nlevel implicitly disables all lower log levels as well.","ref":"Abyss.Logger.html#detach_logger/1"},{"type":"type","title":"Abyss.Logger.log_level/0","doc":"Supported log levels","ref":"Abyss.Logger.html#t:log_level/0"},{"type":"module","title":"Abyss.ServerConfig","doc":"Encapsulates the configuration of a Abyss server instance\n\nThis is used internally by `Abyss.Handler`","ref":"Abyss.ServerConfig.html"},{"type":"function","title":"Abyss.ServerConfig.new/1","doc":"","ref":"Abyss.ServerConfig.html#new/1"},{"type":"type","title":"Abyss.ServerConfig.t/0","doc":"A set of configuration parameters for a Abyss server instance","ref":"Abyss.ServerConfig.html#t:t/0"},{"type":"module","title":"Abyss.Telemetry","doc":"The following telemetry spans are emitted by abyss\n\n## `[:abyss, :listener, *]`\n\nRepresents a Abyss server listening to a port\n\nThis span is started by the following event:\n\n* `[:abyss, :listener, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `local_address`: The IP address that the listener is bound to\n    * `local_port`: The port that the listener is bound to\n    * `transport_module`: The transport module in use\n    * `transport_options`: Options passed to the transport module at startup\n\n\nThis span is ended by the following event:\n\n* `[:abyss, :listener, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `local_address`: The IP address that the listener is bound to\n    * `local_port`: The port that the listener is bound to\n    * `transport_module`: The transport module in use\n    * `transport_options`: Options passed to the transport module at startup\n\n## `[:abyss, :acceptor, *]`\n\nRepresents a Abyss acceptor process listening for connections\n\nThis span is started by the following event:\n\n* `[:abyss, :acceptor, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:listener` which created this acceptor\n\nThis span is ended by the following event:\n\n* `[:abyss, :acceptor, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `connections`: The number of client requests that the acceptor handled\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:listener` which created this acceptor\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:abyss, :acceptor, :spawn_error]`\n\n    Abyss was unable to spawn a process to handle a connection. This occurs when too\n    many connections are in progress; you may want to look at increasing the `num_connections`\n    configuration parameter\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :acceptor, :econnaborted]`\n\n    Abyss was unable to spawn a process to handle a connection since the remote end\n    closed before we could accept it. This usually occurs when it takes too long for your server\n    to start processing a connection; you may want to look at tuning OS-level TCP parameters or\n    adding more server capacity.\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n## `[:abyss, :connection, *]`\n\nRepresents Abyss handling a specific client request\n\nThis span is started by the following event:\n\n* `[:abyss, :connection, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:acceptor` span which accepted\n    this connection\n    * `remote_address`: The IP address of the connected client\n    * `remote_port`: The port of the connected client\n\nThis span is ended by the following event:\n\n* `[:abyss, :connection, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `send_oct`: The number of octets sent on the connection\n    * `send_cnt`: The number of packets sent on the connection\n    * `recv_oct`: The number of octets received on the connection\n    * `recv_cnt`: The number of packets received on the connection\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `parent_telemetry_span_context`: The span context of the `:acceptor` span which accepted\n      this connection\n    * `remote_address`: The IP address of the connected client\n    * `remote_port`: The port of the connected client\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:abyss, :connection, :ready]`\n\n    Abyss has completed setting up the client connection\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :async_recv]`\n\n    Abyss has asynchronously received data from the client\n\n    This event contains the following measurements:\n\n    * `data`: The data received from the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :recv]`\n\n    Abyss has synchronously received data from the client\n\n    This event contains the following measurements:\n\n    * `data`: The data received from the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :recv_error]`\n\n    Abyss encountered an error reading data from the client\n\n    This event contains the following measurements:\n\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :send]`\n\n    Abyss has sent data to the client\n\n    This event contains the following measurements:\n\n    * `data`: The data sent to the client\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :send_error]`\n\n    Abyss encountered an error sending data to the client\n\n    This event contains the following measurements:\n\n    * `data`: The data that was being sent to the client\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :sendfile]`\n\n    Abyss has sent a file to the client\n\n    This event contains the following measurements:\n\n    * `filename`: The filename containing data sent to the client\n    * `offset`: The offset (in bytes) within the file sending started from\n    * `bytes_written`: The number of bytes written\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :sendfile_error]`\n\n    Abyss encountered an error sending a file to the client\n\n    This event contains the following measurements:\n\n    * `filename`: The filename containing data that was being sent to the client\n    * `offset`: The offset (in bytes) within the file where sending started from\n    * `length`: The number of bytes that were attempted to send\n    * `error`: A description of the error\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n\n* `[:abyss, :connection, :socket_shutdown]`\n\n    Abyss has shutdown the client connection\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `way`: The direction in which the socket was shut down\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span","ref":"Abyss.Telemetry.html"},{"type":"type","title":"Abyss.Telemetry.metadata/0","doc":"","ref":"Abyss.Telemetry.html#t:metadata/0"},{"type":"function","title":"Abyss.Telemetry.monotonic_time/0","doc":"","ref":"Abyss.Telemetry.html#monotonic_time/0"},{"type":"type","title":"Abyss.Telemetry.span_name/0","doc":"","ref":"Abyss.Telemetry.html#t:span_name/0"},{"type":"type","title":"Abyss.Telemetry.t/0","doc":"","ref":"Abyss.Telemetry.html#t:t/0"},{"type":"behaviour","title":"Abyss.Transport","doc":"This module describes the behaviour required for Abyss to interact\nwith low-level sockets. It is largely internal to Abyss, however users\nare free to implement their own versions of this behaviour backed by whatever\nunderlying transport they choose. Such a module can be used in Abyss\nby passing its name as the `transport_module` option when starting up a server,\nas described in `Abyss`.","ref":"Abyss.Transport.html"},{"type":"type","title":"Abyss.Transport.address/0","doc":"A socket address","ref":"Abyss.Transport.html#t:address/0"},{"type":"callback","title":"Abyss.Transport.close/1","doc":"Closes the given socket.","ref":"Abyss.Transport.html#c:close/1"},{"type":"callback","title":"Abyss.Transport.controlling_process/2","doc":"Transfers ownership of the given socket to the given process. This will always\nbe called by the process which currently owns the socket.","ref":"Abyss.Transport.html#c:controlling_process/2"},{"type":"callback","title":"Abyss.Transport.getopts/2","doc":"Gets the given options on the socket.","ref":"Abyss.Transport.html#c:getopts/2"},{"type":"callback","title":"Abyss.Transport.getstat/1","doc":"Returns stats about the connection on the socket.","ref":"Abyss.Transport.html#c:getstat/1"},{"type":"callback","title":"Abyss.Transport.listen/2","doc":"Create and return a listener socket bound to the given port and configured per\nthe provided options.","ref":"Abyss.Transport.html#c:listen/2"},{"type":"type","title":"Abyss.Transport.listen_options/0","doc":"A listener socket options","ref":"Abyss.Transport.html#t:listen_options/0"},{"type":"type","title":"Abyss.Transport.listener_socket/0","doc":"A listener socket used to wait for connections","ref":"Abyss.Transport.html#t:listener_socket/0"},{"type":"callback","title":"Abyss.Transport.negotiated_protocol/1","doc":"Returns the protocol negotiated as part of handshaking. Most typically this is via TLS'\nALPN or NPN extensions. If the underlying transport does not support protocol negotiation\n(or if one was not negotiated), `{:error, :protocol_not_negotiated}` is returned","ref":"Abyss.Transport.html#c:negotiated_protocol/1"},{"type":"type","title":"Abyss.Transport.on_close/0","doc":"The return value from a close/1 call","ref":"Abyss.Transport.html#t:on_close/0"},{"type":"type","title":"Abyss.Transport.on_controlling_process/0","doc":"The return value from a controlling_process/2 call","ref":"Abyss.Transport.html#t:on_controlling_process/0"},{"type":"type","title":"Abyss.Transport.on_getopts/0","doc":"The return value from a getopts/2 call","ref":"Abyss.Transport.html#t:on_getopts/0"},{"type":"type","title":"Abyss.Transport.on_listen/0","doc":"The return value from a listen/2 call","ref":"Abyss.Transport.html#t:on_listen/0"},{"type":"type","title":"Abyss.Transport.on_negotiated_protocol/0","doc":"The return value from a negotiated_protocol/1 call","ref":"Abyss.Transport.html#t:on_negotiated_protocol/0"},{"type":"type","title":"Abyss.Transport.on_open/0","doc":"The return value from a open/2 call","ref":"Abyss.Transport.html#t:on_open/0"},{"type":"type","title":"Abyss.Transport.on_peername/0","doc":"The return value from a peername/1 call","ref":"Abyss.Transport.html#t:on_peername/0"},{"type":"type","title":"Abyss.Transport.on_recv/0","doc":"The return value from a recv/3 call","ref":"Abyss.Transport.html#t:on_recv/0"},{"type":"type","title":"Abyss.Transport.on_send/0","doc":"The return value from a send/2 call","ref":"Abyss.Transport.html#t:on_send/0"},{"type":"type","title":"Abyss.Transport.on_setopts/0","doc":"The return value from a setopts/2 call","ref":"Abyss.Transport.html#t:on_setopts/0"},{"type":"type","title":"Abyss.Transport.on_sockname/0","doc":"The return value from a sockname/1 call","ref":"Abyss.Transport.html#t:on_sockname/0"},{"type":"type","title":"Abyss.Transport.on_upgrade/0","doc":"The return value from a upgrade/2 call","ref":"Abyss.Transport.html#t:on_upgrade/0"},{"type":"callback","title":"Abyss.Transport.peername/1","doc":"Returns information in the form of `t:socket_info()` about the remote end of the socket.","ref":"Abyss.Transport.html#c:peername/1"},{"type":"callback","title":"Abyss.Transport.recv/3","doc":"Returns available bytes on the given socket. Up to `num_bytes` bytes will be\nreturned (0 can be passed in to get the next 'available' bytes, typically the\nnext packet). If insufficient bytes are available, the function can wait `timeout`\nmilliseconds for data to arrive.","ref":"Abyss.Transport.html#c:recv/3"},{"type":"type","title":"Abyss.Transport.recv_data/0","doc":"The return data format of :gen_udp.recv","ref":"Abyss.Transport.html#t:recv_data/0"},{"type":"callback","title":"Abyss.Transport.send/2","doc":"Sends the given data (specified as a binary or an IO list) on the given socket.","ref":"Abyss.Transport.html#c:send/2"},{"type":"callback","title":"Abyss.Transport.setopts/2","doc":"Sets the given options on the socket. Should disallow setting of options which\nare not compatible with Abyss","ref":"Abyss.Transport.html#c:setopts/2"},{"type":"type","title":"Abyss.Transport.socket/0","doc":"A socket representing a client connection","ref":"Abyss.Transport.html#t:socket/0"},{"type":"type","title":"Abyss.Transport.socket_get_options/0","doc":"Options which can be set on a socket via setopts/2 (or returned from getopts/1)","ref":"Abyss.Transport.html#t:socket_get_options/0"},{"type":"type","title":"Abyss.Transport.socket_info/0","doc":"Information about an endpoint, either remote ('peer') or local","ref":"Abyss.Transport.html#t:socket_info/0"},{"type":"type","title":"Abyss.Transport.socket_set_options/0","doc":"Options which can be set on a socket via setopts/2 (or returned from getopts/1)","ref":"Abyss.Transport.html#t:socket_set_options/0"},{"type":"type","title":"Abyss.Transport.socket_stats/0","doc":"Connection statistics for a given socket","ref":"Abyss.Transport.html#t:socket_stats/0"},{"type":"callback","title":"Abyss.Transport.sockname/1","doc":"Returns information in the form of `t:socket_info()` about the local end of the socket.","ref":"Abyss.Transport.html#c:sockname/1"},{"type":"module","title":"Abyss.Transport.UDP","doc":"Defines a `Abyss.Transport` implementation based on clear UDP sockets\nas provided by Erlang's `:gen_udp` module. For the most part, users of Thousand\nIsland will only ever need to deal with this module via `transport_options`\npassed to `Abyss` at startup time. A complete list of such options\nis defined via the `t::gen_udp.open_option/0` type. This list can be somewhat\ndifficult to decipher; by far the most common value to pass to this transport\nis the following:\n\n* `ip`:  The IP to listen on. Can be specified as:\n  * `{1, 2, 3, 4}` for IPv4 addresses\n  * `{1, 2, 3, 4, 5, 6, 7, 8}` for IPv6 addresses\n  * `:loopback` for local loopback\n  * `:any` for all interfaces (i.e.: `0.0.0.0`)\n  * `{:local, \"/path/to/socket\"}` for a Unix domain socket. If this option is used,\n    the `port` option *must* be set to `0`\n\nUnless overridden, this module uses the following default options:\n\n```elixir\nreuseaddr: true\n```\n\nThe following options are required for the proper operation of Abyss\nand cannot be overridden:\n\n```elixir\nmode: :binary,\nactive: false\n```","ref":"Abyss.Transport.UDP.html"},{"type":"function","title":"Abyss.Transport.UDP.open/2","doc":"","ref":"Abyss.Transport.UDP.html#open/2"},{"type":"function","title":"Abyss.Transport.UDP.recv/2","doc":"","ref":"Abyss.Transport.UDP.html#recv/2"},{"type":"function","title":"Abyss.Transport.UDP.send/3","doc":"","ref":"Abyss.Transport.UDP.html#send/3"},{"type":"function","title":"Abyss.Transport.UDP.send/4","doc":"","ref":"Abyss.Transport.UDP.html#send/4"},{"type":"function","title":"Abyss.Transport.UDP.send/5","doc":"","ref":"Abyss.Transport.UDP.html#send/5"},{"type":"extras","title":"Abyss","doc":"# Abyss\n\n[![release](https://github.com/gsmlg-dev/abyss/actions/workflows/release.yml/badge.svg)](https://github.com/gsmlg-dev/abyss/actions/workflows/release.yml) \n[![Hex.pm](https://img.shields.io/hexpm/v/abyss.svg)](https://hex.pm/packages/abyss) \n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/abyss)\n\n---","ref":"readme.html"},{"type":"extras","title":"Installation - Abyss","doc":"The package can be installed by adding `abyss` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:abyss, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Run Example - Abyss","doc":"```shell\n# run service\nmix run --no-halt -e 'Code.require_file(\"example/echo.ex\"); Abyss.Logger.attach_logger(:trace); Abyss.start_link(handler_module: Echo, port: 1234); Process.sleep(3600_000); '\n\nmix run --no-halt -e 'Abyss.Logger.attach_logger(:trace); Abyss.start_link(handler_module: Abyss.Echo, port: 1234); Process.sleep(300_000); '\n\n# test\nwhile true\ndo \n  echo \"Hello, UDP $(date +%T)\" | nc -4 -u -w1 127.0.0.1 1234\ndone\n```","ref":"readme.html#run-example"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}